@@ -1,5 +1,7 @@
 #!/usr/bin/env python3
 # coding=utf-8
+import os
+
 import random
 import re
 import string
@@ -14,6 +16,7 @@ KEY_UE_TA_POINTS = 'ta_points'
 KEY_UE_ATTENDED = 'attended'
 KEY_UE_REMARKS = 'remarks'
 KEY_UE_FEEDBACK = 'feedback'
+KEY_UE_GRADING = 'grading'
 KEY_UE_PRE_EVAL_EX = 'pre_eval_ex_%02d'
 
 TAG_ABGABE = 'abgabe_%d'
@@ -68,7 +71,7 @@ def attendance_csv_fieldnames(ep2: Ep2, ue: int) -> [str]:
 
 def pre_eval_csv_fieldnames(ep2: Ep2, ue: int) -> [str]:
     sub_exercises = [KEY_UE_PRE_EVAL_EX % d for d in range(1, ep2.tests_for_exercise(ue) + 1)]
-    return [KEY_STUDENT_ID, KEY_UE_ATTENDED, KEY_UE_REMARKS, KEY_UE_FEEDBACK] + sub_exercises
+    return [KEY_STUDENT_ID, KEY_UE_ATTENDED, KEY_UE_REMARKS, KEY_UE_FEEDBACK, KEY_UE_GRADING] + sub_exercises
 
 
 def pre_eval_csv_fieldnames_ta():
@@ -104,7 +107,7 @@ def checkout(ctx, group, ue: int):
 
     uebung_path = os.path.join(ep2.config.get('Local', 'GitHome'), "uebung")
 
-    errors = False
+    errors = []
 
     with open(students_file, 'r') as infile:  # read students and create entries in attendance file
         reader = csv.DictReader(infile, students_csv_fieldnames(), KEY_INVALID, strict=True)
@@ -127,40 +130,18 @@ def checkout(ctx, group, ue: int):
             repo = ep2.ue_repo(student_id)
 
             # clone or pull repo
-            if not ep2.clone_or_update(os.path.join(uebung_path, student_id), repo, TAG_ABGABE % ue):
-                errors = True
+            clone_result = ep2.clone_or_update(os.path.join(uebung_path, student_id), repo, TAG_ABGABE % ue)
+            if clone_result.is_err():
+                errors += [clone_result.value]
 
         infile.seek(0)
         next(reader, None)  # skip header row
 
-        # if not os.path.exists(att_csv_file):
-        #     f_info.open_write(att_csv_file, True)
-        #     with open(att_csv_file, 'w') as outfile:
-        #         writer = csv.DictWriter(outfile, fieldnames=attendance_csv_fieldnames(ep2, ue), lineterminator='\n')
-        #
-        #         writer.writeheader()
-        #
-        #         for row in reader:
-        #             # no validation required, as the file is the same
-        #
-        #             student_id = row[KEY_STUDENT_ID]
-        #
-        #             writer.writerow({KEY_STUDENT_ID: student_id, KEY_UE_ATTENDED % ue: '0'})
-        #
-        # infile.seek(0)
-        # next(reader, None)  # skip header row
-
         if not os.path.exists(pre_eval_file):
-            if curr_ue_type == ExerciseType.Normal:
-                # writer.writerow({KEY_STUDENT_ID: student_id, KEY_UE_KREUZERL % ue: '_', KEY_UE_REMARKS %
-                # ue: '', KEY_UE_FEEDBACK % ue: ''})
-                click.secho('Pre evaluation files for normal exercises should be created by automatic testing '
-                            'system, please pull tutor repo first', fg='yellow')
 
             f_info.open_write(pre_eval_file, True)
             with open(pre_eval_file, 'w') as outfile:
                 if curr_ue_type == ExerciseType.Normal:
-
                     writer = csv.DictWriter(outfile, fieldnames=pre_eval_csv_fieldnames(ep2, ue), lineterminator='\n')
                 elif curr_ue_type == ExerciseType.Team:
                     writer = csv.DictWriter(outfile, fieldnames=pre_eval_csv_fieldnames_ta(), lineterminator='\n')
@@ -175,13 +156,20 @@ def checkout(ctx, group, ue: int):
 
                     student_id = row[KEY_STUDENT_ID]
 
-                    writer.writerow(
-                        {KEY_STUDENT_ID: student_id, KEY_UE_TA_POINTS % ue: '_', KEY_UE_REMARKS % ue: '',
-                         KEY_UE_FEEDBACK % ue: '', KEY_UE_ATTENDED: 0})
+                    if curr_ue_type == ExerciseType.Normal:
+                        sub_exercises = {KEY_UE_PRE_EVAL_EX % d: '0/0' for d in range(1, ep2.tests_for_exercise(ue) + 1)}
+                        writer.writerow({KEY_STUDENT_ID: student_id, KEY_UE_GRADING: '_', KEY_UE_REMARKS: '', KEY_UE_FEEDBACK: '', KEY_UE_ATTENDED: 0, **sub_exercises})
+                    elif curr_ue_type == ExerciseType.Team:
+                        writer.writerow(
+                            {KEY_STUDENT_ID: student_id, KEY_UE_TA_POINTS: '_', KEY_UE_REMARKS: '',
+                             KEY_UE_FEEDBACK: '', KEY_UE_ATTENDED: 0})
 
     if errors:
         click.secho('with errors', fg='yellow')
 
+        for error in errors:
+            click.secho('\t' + error, fg='yellow')
+
     f_info.print_info()
     click.echo('To grade an exercise run ' + click.style('%s grade' % EVAL_COMMAND_NAME, bold=True) + '.')
     click.echo(
@@ -244,17 +232,15 @@ def grade(ctx, group, ue: int, student, grading, student_feedback, solution_rema
     pre_eval_csv_file = group.pre_eval_csv(ue)  # file for reading
     tmp_file = pre_eval_csv_file + '.tmp'  # file for writing
 
-    if 0 < tasks != len(grading):
-        raise click.BadParameter('number of grades does not match task count')
-
     curr_ue_type = ue_type(ue)
     if curr_ue_type == ExerciseType.Team:
         if points is None:
             points = click.prompt('Points', prompt_suffix=' [0-33]: ', type=click.IntRange(0, 33))
     elif curr_ue_type == ExerciseType.Normal:
-        click.secho('This command should not be used with normal exercises! If you need to make changes, please do so '
-                    'manually!', fg='red', nl=True, bold=True)
-        exit(2)
+        if grading is None:
+            grading = click.prompt('Grading')
+            while not re.match('^[+\\-~0]*$', grading):
+                grading = click.prompt('Grading')
     else:
         click.secho('Unknown exercise type %s' % curr_ue_type, fg='red')
         exit(1)
@@ -269,16 +255,15 @@ def grade(ctx, group, ue: int, student, grading, student_feedback, solution_rema
         with open(tmp_file, 'w') as outfile:
 
             if curr_ue_type == ExerciseType.Normal:
-                # reader = csv.DictReader(infile, fieldnames=pre_eval_csv_fieldnames(ep2, ue), strict=True, restkey=KEY_INVALID)
-                #
-                # headers = next(reader, None)
-                # if not validate_headers(headers):
-                #     click.secho('Malformed file: %s. Invalid headers!' % pre_eval_csv_file, fg='red')
-                #     f_info.print_info()
-                #     exit(1)
-                #
-                # writer = csv.DictWriter(outfile, fieldnames=pre_eval_csv_fieldnames(ep2, ue), lineterminator='\n')
-                return
+                reader = csv.DictReader(infile, fieldnames=pre_eval_csv_fieldnames(ep2, ue), strict=True, restkey=KEY_INVALID)
+
+                headers = next(reader, None)
+                if not validate_headers(headers):
+                    click.secho('Malformed file: %s. Invalid headers!' % pre_eval_csv_file, fg='red')
+                    f_info.print_info()
+                    exit(1)
+
+                writer = csv.DictWriter(outfile, fieldnames=pre_eval_csv_fieldnames(ep2, ue), lineterminator='\n')
             else:
                 reader = csv.DictReader(infile, fieldnames=pre_eval_csv_fieldnames_ta(), strict=True,
                                         restkey=KEY_INVALID)
@@ -305,19 +290,18 @@ def grade(ctx, group, ue: int, student, grading, student_feedback, solution_rema
                     exit(1)
 
                 if curr_ue_type == ExerciseType.Normal:
-                    return
-                    # student_id = row[KEY_STUDENT_ID]
-                    # if student_id == student:
-                    #     row[KEY_UE_KREUZERL % ue] = grading
-                    #
-                    #     if solution_remarks is not None:
-                    #         row[KEY_UE_REMARKS % ue] = escape_csv_string(solution_remarks)
-                    #
-                    #     # append feedback and store in csv
-                    #     if student_feedback is not None and student_feedback != '':
-                    #         row[KEY_UE_FEEDBACK % ue] = escape_csv_string(student_feedback)
-                    #
-                    # writer.writerow(row)
+                    student_id = row[KEY_STUDENT_ID]
+                    if student_id == student:
+                        row[KEY_UE_GRADING] = grading
+
+                        if solution_remarks is not None:
+                            row[KEY_UE_REMARKS] = escape_csv_string(solution_remarks)
+
+                        # append feedback and store in csv
+                        if student_feedback is not None and student_feedback != '':
+                            row[KEY_UE_FEEDBACK] = escape_csv_string(student_feedback)
+
+                    writer.writerow(row)
                 else:
                     if team_index is None:
                         click.secho('Team index could not be loaded, make sure teams exist (might need to be created)',
@@ -326,14 +310,14 @@ def grade(ctx, group, ue: int, student, grading, student_feedback, solution_rema
 
                     student_id = row[KEY_STUDENT_ID]
                     if student_id == student and not team_index.skip(student_id):
-                        row[KEY_UE_TA_POINTS % ue] = points
+                        row[KEY_UE_TA_POINTS] = points
 
                         if solution_remarks is not None:
-                            row[KEY_UE_REMARKS % ue] = escape_csv_string(solution_remarks)
+                            row[KEY_UE_REMARKS] = escape_csv_string(solution_remarks)
 
                         # append feedback and store in csv
                         if student_feedback is not None and student_feedback != '':
-                            row[KEY_UE_FEEDBACK % ue] = escape_csv_string(student_feedback)
+                            row[KEY_UE_FEEDBACK] = escape_csv_string(student_feedback)
 
                         writer.writerow(row)
 
@@ -423,7 +407,7 @@ def submit(ctx, group: str, ue: int):
                 stud_data = students[row[KEY_STUDENT_ID]]
 
                 if curr_ue_type == ExerciseType.Normal:
-                    grading = [row[KEY_UE_PRE_EVAL_EX % d] for d in range(1, ep2.tests_for_exercise(ue) + 1)]
+                    grading = row[KEY_UE_GRADING]   # for d in range(1, ep2.tests_for_exercise(ue) + 1)]
                     if any(grade == '_' for grade in grading):
                         click.secho('%s has not been graded yet. aborting!' % row[KEY_STUDENT_ID], fg='red')
                         f_info.print_info()
@@ -431,10 +415,11 @@ def submit(ctx, group: str, ue: int):
                         return
 
                     template.grading = grading
+                    template.tests = [row[KEY_UE_PRE_EVAL_EX % d] for d in range(1, ep2.tests_for_exercise(ue) + 1)]
                     template.points = -1
                 elif curr_ue_type == ExerciseType.Team:
                     points = row[KEY_UE_TA_POINTS]
-                    if template.points == '_':
+                    if points == '_':
                         click.secho('%s has not been graded yet. aborting!' % row[KEY_STUDENT_ID], fg='red')
                         f_info.print_info()
                         exit(1)
@@ -535,12 +520,13 @@ def list_ungraded(ctx, ue, group):
                     exit(1)
 
                 if curr_ue_type == ExerciseType.Normal:
-                    test_suites = [KEY_UE_PRE_EVAL_EX % d for d in range(1, ep2.tests_for_exercise(ue) + 1)]
-                    if any(row[suite] == '_' for suite in test_suites):  # if not evaluated => list
+                    grading = row[KEY_UE_GRADING]
+                    if any(grade == '_' for grade in grading):  # if not evaluated => list
                         click.echo(row[KEY_STUDENT_ID])
                         empty = False
+
                 elif curr_ue_type == ExerciseType.Team:
-                    if row[KEY_UE_TA_POINTS % ue] == '_':  # if not evaluated => list
+                    if row[KEY_UE_TA_POINTS] == '_':  # if not evaluated => list
                         click.echo(row[KEY_STUDENT_ID])
                         empty = False
     except IOError:
