@@ -31,7 +31,7 @@ def ex_test_csv_fieldnames():
 class StudentIndex:
 
     def __init__(self, group: Ep2Group, ue: int, single: bool):
-        current_ue_csv = group.submission_csv(ue)
+        current_ue_csv = group.exercise_test_csv(ue)
 
         self.students = group.student_list()
         self.group = group
@@ -83,7 +83,7 @@ class StudentIndex:
             self.build_lookup_index(ue - 1)
 
     def build_lookup_index(self, prev_ue):
-        previous_ue_csv = self.group.submission_csv(prev_ue)
+        previous_ue_csv = self.group.exercise_test_csv(prev_ue)
 
         done_students = map(lambda x: x[KEY_STUDENT_ID], self.done)
         previous = []
@@ -187,7 +187,7 @@ def submission(ctx, group, ue, late, idea, grade, single):
                    ' to change the grading of a student')
         return
 
-    csv_file = group.submission_csv(ue)
+    csv_file = group.exercise_test_csv(ue)
     csv_path = csv_file[:csv_file.rindex(os.sep)]  # needed to create parent dir
 
     try:
@@ -311,7 +311,7 @@ def checkout(ctx, group, ue: int):
     group = ep2.group(group)
 
     git_path = ep2.config.get("Local", "GitHome")
-    csv_path = group.submission_csv(ue)
+    csv_path = group.exercise_test_csv(ue)
     uebung_path = os.path.join(git_path, "uebung")
 
     try:
@@ -367,7 +367,7 @@ def grade(ctx, repo_owner, points, ue, group, remarks, feedback):
     ep2 = Ep2(verbose=ctx.obj["VERBOSE"])
     group = ep2.group(group)
 
-    csv_file = group.submission_csv(ue)
+    csv_file = group.exercise_test_csv(ue)
     tmp_file = csv_file + ".tmp"  # tmp file for editing
 
     team = None
@@ -450,7 +450,7 @@ def submit(ctx, group, ue):
     click.secho('Challenge %s' % challenge, bold=True)
     click.echo('\n' + ('=' * 30) + '\n')
 
-    ex_test_csv_file = group.submission_csv(ue)
+    ex_test_csv_file = group.exercise_test_csv(ue)
     ex_test_csv_file_tmp = ex_test_csv_file + '.tmp'
 
     students = group.student_list()
@@ -574,7 +574,7 @@ def list_ungraded(ctx, ue, group):
     ep2 = Ep2(verbose=ctx.obj["VERBOSE"])
     group = ep2.group(group)
 
-    csv_file = group.submission_csv(ue)
+    csv_file = group.exercise_test_csv(ue)
     empty = True
 
     try:
